﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ThinkGeo.MapSuite.Core;
using System.Collections.ObjectModel;

namespace Post9837_WPFOverlayDrawWithGeoCanvasParameter
{
    class cCustomLayer : Layer
    {

        public bool ServerCache;

        private ShapeFileFeatureLayer worldLayer;
        private cGeoServer geoServer;

        public cCustomLayer()
        {
            init();
        }

        private void init()
        {
            worldLayer = new ShapeFileFeatureLayer(@"..\..\App_Data\Countries02.shp");
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, GeoColor.FromArgb(100, GeoColor.SimpleColors.Green));
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            geoServer = new cGeoServer();
        }


        protected override void DrawCore(GeoCanvas canvas, System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInAllLayers)
        {

            if (ServerCache)
            {
                DrawOnServer(canvas, labelsInAllLayers); //if caching is activated the map image is drawn on the server
            }
            else
            {
                Collection<SimpleCandidate> tempLabelCandidates = new Collection<SimpleCandidate>();
                lock (worldLayer)
                {
                    if (!worldLayer.IsOpen) worldLayer.Open();
                    worldLayer.Draw(canvas, tempLabelCandidates);

                }
            }

        }

        private void DrawOnServer(GeoCanvas canvas, System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInAllLayers)
        {
            
            //In our productive application geoServer class is working within an web service. Web service draws the image and sends it to the client.
            byte[] byteArray = geoServer.getImage(canvas.CurrentWorldExtent.GetWellKnownText(), (int)canvas.Width, (int)canvas.Height, canvas.MapUnit);

            System.IO.MemoryStream ImageStream = new System.IO.MemoryStream(byteArray);
            GeoImage GeoImage = new GeoImage(ImageStream);

            PointShape centerPoint = canvas.CurrentWorldExtent.GetCenterPoint();
            ScreenPointF screenPoint = ExtentHelper.ToScreenCoordinate(canvas.CurrentWorldExtent, centerPoint, canvas.Width, canvas.Height);

            canvas.DrawScreenImage(GeoImage, screenPoint.X, screenPoint.Y, canvas.Width, canvas.Height, DrawingLevel.LevelOne, 0, 0, 0);
            ImageStream.Close();
        }

    }
}
